      subroutine main_loop
c
c-----Description-------------------------------------------------------
c
c Author/Date: Mathias Bavay  11/04
c      Modifications:
c
c ----------------------------------------------------------------------
c
c Purpose: This subroutines performs the calculations. 
c     
c Called by: Program ZDEM
c    
c Include the files with the various keywords and integer flags.
c
      include 'zdemparm.h'
      include 'zdempprm.h'
c
c Include the files specifying the array dimensions and the common blocks.
c
      include 'zdemmax.h'
      include 'zdemcomm.h'
      include 'zdemwork.h'
      include 'zdemout.h'
      include 'zdemenv.h'
      include 'zdemvars.h'
      include 'zdemloop.h'

c
c Include file with version string
c
      include 'version.h'
      
c----Buffer variable for C++ output-----------------------------------------
      character(len=80)::buffer
c---------------------------------------------------------------------------

c Print out time step to monitor run progress
c
          iptold=iptime
          if (tmax .le. 100.0e-6) then
             iptime=tim*1.0e9
             if (tmax .gt. 10.0e-6) then
                jdiv=2000
             else
                jdiv=200
             endif
             if ((mod(iptime,jdiv).eq.0) .and.
     &           (iptold .ne. iptime)) then
c             write (6, 212) iptime
             write (buffer, 212) iptime
	     call writebuffer(buffer)
             end if
  212        format (' Time = ',i5,' ns')
          else if ((tmax.gt.100.0e-6).and.(tmax.le.100.0e-3)) then
             iptime=tim*1.0e6
             if (tmax .gt. 10.0e-3) then
                jdiv=2000
             else
                jdiv=200
             endif
             if ((mod(iptime,jdiv).eq.0) .and.
     &           (iptold .ne. iptime)) then
             write (buffer, 220) iptime
c             write (6, 220) iptime
	     call writebuffer(buffer)
             end if
  220        format (' Time = ',i5,' microsec')
          else
             iptime=tim*1.0e3
             if (tmax .gt. 10.0) then
                jdiv=2000
             else
                jdiv=200
             endif
             if ((mod(iptime,jdiv).eq.0) .and.
     &           (iptold .ne. iptime)) then
             write (buffer, 221) iptime
c             write (6, 221) iptime
	     call writebuffer(buffer)
             end if
  221        format (' Time = ',i5,' msec')
          endif

c
c
c branch connections - load bb,cc matrices
c Note that the do 211 loop will not be executed if nb=1 .
c
        nelmt = nbe(nb)
        do i = 1, nelmt
          bb(i) = 0.0
          cc(i) = 0.0
        end do
c        nbm = nb - 1   This line removed since nbm already defined!!!!
        do 211 icb = 1, nbm
          icbp       = icb + 1
          node_num   = indexb(1,icb)
          iexit_type = indexb(2,icb)
          icx        = (node_num-2)*k + 1 + nbe(icb)
          icy        = (node_num-1)*k + 1 + nbe(icb)
c
c Fill for top or end branch.
c
          if (iexit_type .eq. topbranch) then
            zib(node_num-1,1) = -zir(1,icbp)
            zib(node_num,1)   = -zib(node_num-1,1)
            bb(icx+1)         = +0.5
            bb(icy+1)         = -0.5
            vsour(icb)        = v(node_num-1,1) - v(node_num,1)
            cc(icx)           = -1.0
            cc(icy)           = +1.0
          else
            zib(node_num,1) = -zir(1,icbp)
            bb(icy+1)       = +0.5
            vsour(icb)      = v(node_num,1)
            cc(icy)         = -1.0
          end if
  211   continue
c
c Reset variable element values.
c
        itab_counter=0
c------------------------------------------------------------------------
c
	call get_model
c
c------------------------------------------------------------------------
c
c Reset MITL conductances
c
        do imitl = 1, nmitline
          if (indexmitl(5,imitl) .eq. mitline) then
            call reset_mitl (imitl)
          else if (indexmitl(5,imitl) .eq. pmitline) then
            call reset_pmitl (imitl)
          end if
        end do
c
c -----------------------------------------------------------------------------
c
c Load the 'a' vector using the voltage and current equations at each node.
c
c A note about entering zero values:  There is a divide by certain 'a' elements
c in the matrix solver, hence these elements may not be zero.  These elements
c are:
c   For the current equations:  First and last nodes in a branch: a(ny+3)
c                               All other nodes in a branch:      a(ny+4)
c   For the voltage equations:  First and last nodes in a branch: a(ny+4)
c                               All other nodes in a branch:      a(ny+3)
c
        k    = 2
        nrow = 3*k + 1
c
c Set the total number of elements in a.
c Then zero it.
c
        nele = nrow * k * ntot
        do i = 1, nele
          a(i) = 0.0
        end do
c
c Now enter the appropriate values into a.
c
c
c First, loop over the branches.
c
        do 102   ib = 1, nb
          nrx       = nr(ib)
          nzz       = nrow * k * nadd_array(ib)
          nrm       = nrx - 1
c
c Loop over the nodes, from node 2 to the next-to-last using the general
c voltage and current equations.
c
          do 5 i = 2, nrm
c
c Current equation, second row.  a(ny+4) is the diagonal element
c and is never zero.
c
            j       = (i-1)*2 + 1
            ny      = nrow*j + nzz
c this is the -1 that always preceeds the AVi and the AIi in the row
            a(ny+2) = -0.5
c a(ny+3) loads AVi, in this case it was not scaled by 2X as in the manual
c rht is the inverse time step
            a(ny+3) = rht*c(i,ib) + 0.5*g(i,ib)
c this is the +1 that always follows the AVi and the AIi in the row
            a(ny+4) = +0.5
c loads the value for BVi
            a(ny+7) = (rht*c(i,ib) - 0.5*g(i,ib)) * v(i,ib)
     &                - 0.5 * (zir(i,ib)-zir(i-1,ib)) + 0.5*zib(i,ib)
c
c Voltage equation, first row.  a(ny+3) is the diagonal element and is
c never zero.
c
            ny      = ny - nrow
c this is the -1 that always preceeds the AVi and the AIi in the row 
            a(ny+3) = -0.5
c loads AIi, it is not scaled by 2x
            a(ny+4) = rht*zlr(i,ib) + 0.5*rr(i,ib)
c this is the +1 that always follows the AVi and the AIi in the row
            a(ny+5) = +0.5
c loads the value for BIi
            a(ny+7) = (rht*zlr(i,ib) - 0.5*rr(i,ib)) * zir(i,ib)
     &                + 0.5*(v(i,ib)-v(i+1,ib))
c loops to load up all of the nodes except the first and last nodes.
    5     continue
c
c Do the first and last nodes of each branch separately because of
c boundary conditions and the possibility of sources.
c
          nr2 = 2 * nrx
c
c First row.  Current equation.  a(ny+3) is the diagonal element and
c should not be zero.
c
          ny  = nzz
c
c If this is the first block, look for voltage and current sources.
c iblock sets the block type.
c
          if (ib .eq. 1) then
            iblock1 = iin(1,1,1)
c
c Voltage source for block 1 in branch 1 (current equation).
c Set the voltage at the new time.
c Note: that we are assuming that at most one voltage source block is
c present and that it is block 1 of branch 1.
c
            if (iblock1 .eq. voltsource) then
              call set_voltage (tim, 1, vtime)
              a(ny+3) = +1.0
              a(ny+7) = vtime
c
c Current source for block 1 in branch 1 (current equation).
c Note: that we are assuming that at most one current source block is
c present and that it is block 1 of branch 1.
c Here we set the voltage at node 1 = 0.
c
            else if (iblock1 .eq. currsource) then
              a(ny+3) = +1.0
c
c Anything else at the beginning (current equation).
c
            else
              any3    = 0.5*g(1,1) + rht*c(1,1)
              if (abs(any3) .lt. 1.0e-6) then
                a(ny+3) = 1.0e-6
              else
                a(ny+3) = any3
              end if
              a(ny+4) = +0.5
              a(ny+7) = (-0.5*g(1,1) + rht*c(1,1))*v(1,1) - 0.5*zir(1,1)
c
            end if
c
c First node for all but first branch (current equation).
c Set the voltage at the first node = 0, the branch connection is
c made elsewhere.
c
          else
            a(ny+3) = +1.0
c
          end if
c
c Second row (voltage equation) for first node. a(ny+4) is the diagonal
c element and may not fall below 1e-6.
c Only need to modify the usual equation if we have a current source
c (as the first block in branch 1.
c
          ny = nrow + nzz
c
c Current source, set the current at this time: I1 = I(t).
c
          if ((ib .eq. 1) .and. (iin(1,1,1) .eq. currsource)) then
            call set_current (tim, 1, ctime)
            a(ny+4) = +1.0
            a(ny+7) = ctime
c
c Every other block type.
c
          else
            a(ny+3) = -0.5
            any4    = rht*zlr(1,ib) + 0.5*rr(1,ib)
            if (abs(any4) .lt. 1.0e-6) then
              a(ny+4) = 1.0e-6
            else
              a(ny+4) = any4
            end if
            a(ny+5) = +0.5
            a(ny+7) = (rht*zlr(1,ib) - 0.5*rr(1,ib)) * zir(1,ib)
     &                + 0.5*(v(1,ib)-v(2,ib))
c
          end if
c
c First row for last node.  Current equation.  a(ny+3) may not fall below
c 1e-6.  Check for voltage or current source at end or normal termination.
c
          ny = (nr2-2)*nrow + nzz
c
c Check the end of branch index which indicates a source or no source.
c If it is greater then zero, it is the index for the source.
c
          if_vendsource = ivbranch_end(ib)
          if_cendsource = icbranch_end(ib)
c
c Voltage source
c
          if (if_vendsource .gt. 0) then
            call set_voltage (tim, if_vendsource, vtime)
            a(ny+3) = +1.0
            a(ny+7) = vtime
c
c Normal termination, any other block including a current source (cendsource).
c
          else
            a(ny+2) = -0.5
            any3    = rht*c(nrx,ib) + 0.5*g(nrx,ib)
            if (abs(any3) .lt. 1.0e-6) then
              a(ny+3) = 1.0e-6
            else
              a(ny+3) = any3
            end if
            a(ny+4) = +0.5
            a(ny+7) = (rht*c(nrx,ib) - 0.5*g(nrx,ib)) * v(nrx,ib)
     &                - 0.5*(zir(nrx,ib)-zir(nrx-1,ib))
          end if
c
c Second row for last node, voltage equation. a(ny+4) can not be zero.
c Set the current flowing in or out of the branch.
c
          ny = (nr2-1)*nrow + nzz
c
c Current source and SCL current source.
c
          if (if_cendsource .gt. 0) then
            itypcs = itypcend(if_cendsource)
            if (itypcs .eq. cendsource) then
              call set_current (tim, if_cendsource, ctime)
              a(ny+4) = +1.0
              a(ny+7) = ctime
            else if (itypcs .eq. csclsource) then
              vcond   = v(nrx,ib)
              call set_sclcurr (tim, vcond, if_cendsource, ctime)
              a(ny+4) = +1.0
              a(ny+7) = ctime
            end if
c
c Termination without a current source, set for no current in the branch.
c
          else
            a(ny+4) = 1.0
          end if
            
c
  102   continue
c
c
c invert the matrix to find the new voltages and currents.
c
        k    = 2
        n    = ntot
        nk   = n*k
        nm1  = n-1
        m    = 3*k+1
        km   = k*m
        kmm  = k-m
        k2p1 = k*2+1
        mp1  = m+1
        nkm  = n*k*m
        nkp1 = nk+1
        k2p2 = k2p1+1
        mm1  = m-1
        km1  = k-1
c
        do 51 i51 = 1, n
          ia   = (i51-1)*km+kmm
          iib  = ia+k2p1
          i1km = i51*km
c
          l1   = 1
          do l = 1, nbm
            if (i51 .gt. nbv(l)) l1=l1+1
          end do
          l2   = (i51-1)*k
c
          do 52 i52 = 1, k
            ia   = ia+mp1
            iib  = iib+m
            ic   = ia
            id   = iib
            ie   = i52+1
            l6   = l2+i52
            l8   = k+k-i52
            l13  = (i51-1)*nrow*k+i52*nrow
            raia = 1.0/a(ia)
            do i53 = ia, iib
              a(i53) = a(i53)*raia
            end do
c
c divide bb along row with pivotal diag element
c
            if ((nb .gt. 1) .and. (i51 .le. nbv(nbm))) then
              do l3 = l1, nbm
                ip     = nbe(l3)+l6
                bb(ip) = raia*bb(ip)
              end do
            end if
c      
c do 54 will not be executed if ie > k (FORTRAN 77)
c
            do 54 i54 = ie, k
              ic = ic+m
              id = id+m
              if (a(ic) .ne. 0.0) then
                aic = a(ic)
                ik  = ia-ic
                do i55 = ic, id
                  if     = i55 + ik
                  a(i55) = a(i55)-a(if)*aic
                end do
c
c modify bb along row for rest of block
c
                if ((nb .gt. 1) .and. (i51 .le. nbv(nbm))) then
                  do l3 = l1, nbm
                    iq     = nbe(l3)+l6
                    ip     = iq+i54-1
                    bb(ip) = bb(ip)-bb(iq)*aic
                  end do
                end if
c
              end if 
   54       continue
c
            if (i51.eq.n) go to 66
            ig = i1km+i52-m
            ih = i1km
            do 56 i56 = 1, k
              ig = ig+m
              ih = ih+m
              ii = ih-k-1
              if (a(ig) .ne. 0.0) then
                l7  = k-i52+i56
                aig = a(ig)
                il  = ia-ig
                do i58 = ig, ii
                  ij     = il+i58
                  a(i58) = a(i58)-a(ij)*aig
                end do
                a(ih) = a(ih)-a(iib)*aig  
c
c modify bb along row for next block down
c
                if ((nb .gt. 1) .and. (i51 .le. nbv(nbm))) then
                  do l3 = l1, nbm
                    iq     = nbe(l3)+l6
                    ip     = iq+l7
                    bb(ip) = bb(ip)-bb(iq)*aig
                  end do
                end if
c
              end if
   56       continue
c
c modify cc matrix here
c
            if ((nb .gt. 1) .and. (i51 .le. nbv(nbm))) then
              do 309 l3 = l1, nbm
                iq  = nbe(l3)+l6
                l10 = nbe(l3)+k
                l11 = nbv(l3)*nrow*k+k+k
                l12 = nbv(l3)*k+1
                if (cc(iq) .ne. 0.0) then
                  cmx = cc(iq)
                  ip  = iq+l8
                  if (ip.gt.nbe(l3+1)) ip=nbe(l3+1)
                  iax = ia-1
                  do l4 = iq, ip
                    iax    = iax+1
                    cc(l4) = cc(l4)-cmx*a(iax)
                  end do
c
c modify cc element with bb above
c
                  l3m = l3-1
                  if (l1 .le. l3m) then
                    do l5 = l1, l3m
                      iq     = nbv(l5)*k+l10
                      ip     = nbe(l5)+l6
                      cc(iq) = cc(iq)-cmx*bb(ip)
                    end do
                  end if
c
c modify a element with bb above
c
                  ip    = nbe(l3)+l6
                  iq    = l11
                  a(iq) = a(iq)-cmx*bb(ip)
c
c modify bb element with bb above
c
                  l3p = l3+1
                  if (l3p .le. nbm) then
                    do l5 = l3p, nbm
                      ip     = nbe(l5)+l6
                      iq     = nbe(l5)+l12
                      bb(iq) = bb(iq)-cmx*bb(ip)
                    end do
                  end if
c
c modify last a column
c
                  ip    = l13
                  iq    = nbv(l3)*nrow*k+nrow
                  a(iq) = a(iq)-cmx*a(ip)
                end if
  309         continue
            end if
c
c
   66       continue
   52     continue
   51   continue
c
        jf    = nkm
        jg    = jf-k
        jh    = jg-1
        x(nk) = a(jf)
c
        do j1 = 2, k
          jg  = jg-mp1
          jh  = jh-m
          jis = nkp1-j1
          ji  = jis
          jl  = jf-m*(j1-1)
          sum = a(jl)
          do j2 = jg, jh
            ji  =ji+1
            sum = sum-a(j2)*x(ji)
          end do
          x(jis) = sum
        end do
c
        do j3 = 1, nm1
          ja  = n-j3
          l14 = (ja-1)*k+k+1
          jb  = ja*km+k2p2
          jc  = ja*km+mm1
          jk  = ja*k+1
c
          do j4 = 1, k
            jb  = jb-mp1
            jc  = jc-m
            jd  = jc+1
            sum = a(jd)
            jes = jk-j4
            je  = jes
c
            do j5 = jb, jc
              je  = je+1
              sum = sum-a(j5)*x(je)
            end do
c
            l1 = n-j3
            if (l1 .le. nbv(nbm)) then
              l2 = 1
              do ib = 1, nb
                if(l1.gt.nbv(ib)) l2=l2+1
              end do
c
              if (l2 .le. nbm) then
                do l3 = l2, nbm
                  iq  = nbe(l3)+l14-j4
                  ip  = (nbv(l3)+1)*k
                  sum = sum-bb(iq)*x(ip)
                end do
              end if
            end if
c
            x(jes) = sum
          end do
        end do
c
c end - invert matrix - soln. matrix x
c
        do ib = 1, nb
          nzz  = nadd_array(ib) * k
          nrx  = nr(ib)
c
          do i = 1, nrx
            j          = (i-1)*2 + 1 + nzz
            vn(i,ib)   = x(j)
            zirn(i,ib) = x(j+1)
          end do
        end do
c

       call energy_checks
       
c
c Fill the file-table-ufo-pff-csv-sfc arrays with the 
c appropriate numbers.
c
        if (numout .gt. 0) then
          call build_out (tim, timehalf, ht, rht)
        end if
c
c If we have reached a printing cycle (icycle = ncycle) then
c print out current status at the half time step.
c
        if (icycle .ge. ncycle) then
          icycle = 0
          call cycle_print (i2, tim-0.5*ht, esour, eind, ecap, econ,
     &                      eres, elossind, elosscap, error)
        end if
c
c Now put the new values of V and I into the old V and I arrays for the
c next iteration.
c
        do ib = 1, nb
          nrx = nr(ib)
          do i = 1, nrx
           vold(i,ib)=v(i,ib)
           v(i,ib)   = vn(i,ib)
           zirold(i,ib)=zir(i,ib)
           zir(i,ib) = zirn(i,ib)
          end do
        end do

      return
      end
      
