       subroutine zdem(ifile,clen)
c -----------------------------------------------------------------------------
c
c    SSSS    CCCC   RRRRR   EEEEEE    AA    M    M  EEEEEE  RRRRR
c   S    S  C    C  R    R  E        A  A   MM  MM  E       R    R
c   S       C       R    R  E       A    A  M MM M  E       R    R
c    SSSS   C       RRRRR   EEEEE   AAAAAA  M MM M  EEEEE   RRRRR
c        S  C       R   R   E       A    A  M    M  E       R   R
c   S    S  C    C  R    R  E       A    A  M    M  E       R    R
c    SSSS    CCCC   R    R  EEEEEE  A    A  M    M  EEEEEE  R    R
c
c -----------------------------------------------------------------------------
c
c     Point of Contact:
c       Mark L. Kiefer
c       Sandia National Laboratories
c       MS-1186
c       PO Box 5800
c       Albuquerque NM 87185-1186
c       505/845-7271 (PH)
c       mlkiefe@sandia.gov (e-mail)
c
c     SCREAMER Copyright:
c       Copyright 1996
c       Sandia Corporation
c       Under the terms of Contract DE-AC04-94AL85000,
c       there is a non-exclusive license for use of this work by
c       or on behalf of the U.S. Government
c
c -----------------------------------------------------------------------------
c
c     series matrix load and solution
c
c     single line code for pulse power design
c     array parameters - nr mesh points - 1 at top
c     v,vn = old, new node voltage
c     zib = current entering branch segment
c     zir,zirn = old, new long. current in R - L segment
c     zlr = series inductance, c = shunt capacitance, 
c     g = shunt conductance, rr = series resistance
c     shunt capacitance at the half time step.
c     inductance (vacuum) = impedance/speed of light
c     capacitance (vacuum) = 1/(impedance*speed of light)
c     capacitance (vacuum) = epso*width*length/height
c     inductance (vacuum) = zmuo*height*length/width
c
c -----------------------------------------------------------------------------
c
c Include the files with the various keywords and integer flags.
c
      include 'zdemparm.h'
      include 'zdempprm.h'
c
c Include the files specifying the array dimensions and the common blocks.
c
      include 'zdemmax.h'
      include 'zdemcomm.h'
      include 'zdemwork.h'
      include 'zdemout.h'
      include 'zdemenv.h'
      include 'zdemvars.h'
      include 'zdemloop.h'
c
c Define passed variables
c
      character*(*) ifile
      integer       clen
c
c Define internal variables
c
      integer  ibgin
      integer  status


c---------------------------------------------------------
c INITIALIZATIONS

c Clear arrays
      call clrarry
      
c print a start banner (date, time, version...)
      call banner_start_run(bgin)
      
c Get environment variables
      call get_env_var(ifile,clen)
c Read in the SCREAMER input file (assigned to logical unit 4)
c This subroutine also gives a "nice" print-out of the data.
      call readscreamerdata(status)
      if (status .ne. 0) then
        goto 802
      endif
c Echo the set-up showing all values (slightly cryptic, but useful
c for debugging).
      if (echoset .eq. yes_echo) then
        call echo_setup
      end if

c initialize the problem
      call init_problem(status)
      if (status .ne. 0) then
        call close_outfile(iunit,ierr)
        goto 802
      endif

c---------------------------------------------------------
c CALCULATIONS
c   First, put out a banner page for the status cycle printing for t > 0.
c
      call banner (cyclln)
c ***** Loop over all time steps. *****
c   tim is the problem time and timehalf is the current half-step time.
c   icycle is a counter for printout of current circuit status.
      do 2 i2 = 1, nsteps
        tim        = i2 * ht
        timehalf   = tim - htd2
        icycle     = icycle + 1
        call main_loop
    2 continue      

c---------------------------------------------------------
c OUTPUTS and END
      call close_outfile(iunit,ierr)
c ***** Now do the output requests if we executed all cycles.
       call write_outs
  802  continue
       call banner_end_run(ifile,clen,bgin)

      end subroutine zdem
